#!/usr/bin/python
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Helper functions for fixers."""

from collections.abc import MutableMapping
from debmutate.control import ControlEditor
from debian.deb822 import Deb822
import sys
from typing import Optional, Tuple, Union, List

from . import (
    DEFAULT_MINIMUM_CERTAINTY,
    certainty_sufficient,
)
from .lintian_overrides import (
    get_overrides,
    LintianOverride,
    load_renamed_tags,
)


from debian.changelog import Version
import os


class LintianIssue(object):
    """Represents a lintian issue."""

    target: Tuple[str, Optional[str]]
    info: Optional[Union[str, Tuple[str, ...]]]
    tag: str

    def __init__(
        self,
        target: Union[MutableMapping, Tuple[str, str], str],
        tag: str,
        info: Optional[Union[str, Tuple[str, ...]]] = None,
    ):
        if isinstance(target, MutableMapping):
            if "Source" in target:
                target = ("source", target["Source"])
            elif "Package" in target:
                target = ("binary", target["Package"])
            else:
                raise ValueError(
                    "unable to determine source/binary package from target"
                )
        elif target == "source":
            target = ("source", None)
        if isinstance(info, tuple):
            info = " ".join(info)
        self.target = target
        self.info = info
        self.tag = tag

    def override_exists(self):
        return _override_exists(
            tag=self.tag, info=self.info, type=self.target[0], package=self.target[1]
        )

    def should_fix(self):
        if self.override_exists():
            _overriden_issues.append(self)
            return False
        return True

    def report_fixed(self):
        _fixed_lintian_issues.append(self)

    def __str__(self):
        ret = []
        if self.target[1] is not None:
            ret.append(self.target[1] + " ")
        ret.append(self.target[0])
        ret.append(": " + self.tag + (' ' + self.info) if self.info else '')
        return "".join(ret)

    def __repr__(self):
        return "%s(target=%r, tag=%r, info=%r)" % (
            type(self).__name__,
            self.target,
            self.tag,
            self.info,
        )


_fixed_lintian_issues: List[LintianIssue] = []
_present_overrides: Optional[List[LintianOverride]] = None
_overriden_issues: List[LintianIssue] = []
_tag_renames = None


def _override_exists(
    tag: str,
    info: Optional[str] = None,
    package: Optional[str] = None,
    type: Optional[str] = None,
    arch: Optional[str] = None,
) -> bool:
    global _present_overrides, _tag_renames
    if _present_overrides is None:
        _present_overrides = list(get_overrides())
    if not _present_overrides:
        return False
    if _tag_renames is None:
        _tag_renames = load_renamed_tags()
    for override in _present_overrides:
        if _tag_renames.get(override.tag) == tag:
            tag = override.tag
        if override.matches(package=package, info=info, tag=tag, arch=arch, type=type):
            return True
    return False


def fixed_lintian_tags():
    return set([issue.tag for issue in _fixed_lintian_issues])


def fixed_lintian_tag(
    target: Union[MutableMapping, Tuple[str, str], str],
    tag: str,
    info: Optional[Union[str, Tuple[str, ...]]] = None,
):
    """Register a lintian tag as being fixed."""
    LintianIssue(target, tag, info).report_fixed()


def reset() -> None:
    """Reset any global state that may exist."""
    global _fixed_lintian_issues, _present_overrides, _overriden_issues
    _fixed_lintian_issues = []
    _present_overrides = None
    _overriden_issues = []


def report_result(description=None, certainty=None, patch_name=None):
    """Report the result of a fixer.

    Args:
      description: Description of the fix
      certainty: Certainty of the fix
      patch_name: Suggested patch name, if there are upstream changes
    """
    if description:
        print(description)
    if certainty:
        print("Certainty: %s" % certainty)
    fixed_lintian_tags = set(
        [issue.tag for issue in _fixed_lintian_issues])
    if fixed_lintian_tags:
        print("Fixed-Lintian-Tags: %s" % ", ".join(sorted(fixed_lintian_tags)))
    if _overriden_issues:
        print("Overridden-Lintian-Issues:")
        for issue in _overriden_issues:
            print(' ' + str(issue))
    if patch_name:
        print("Patch-Name: %s" % patch_name)
    reset()


def net_access_allowed():
    """Check whether network access is allowed."""
    return os.environ.get("NET_ACCESS", "disallow") == "allow"


def compat_release():
    """Codename of oldest release to stay compatible with."""
    return os.environ.get("COMPAT_RELEASE", "sid")


def upgrade_release():
    """Codename of oldest release to allow upgrading from."""
    return os.environ.get("UPGRADE_RELEASE", "oldstable")


def current_package_version():
    return Version(os.environ["CURRENT_VERSION"])


def package_is_native():
    return not current_package_version().debian_revision


def meets_minimum_certainty(certainty):
    return certainty_sufficient(
        certainty, os.environ.get("MINIMUM_CERTAINTY", DEFAULT_MINIMUM_CERTAINTY)
    )


def trust_package():
    return os.environ.get("TRUST_PACKAGE") == "true"


def opinionated():
    return os.environ.get("OPINIONATED", "no") == "yes"


def warn(msg):
    sys.stderr.write("%s\n" % msg)


def diligence():
    return int(os.environ.get("DILIGENCE", "0"))


def source_package_name():
    return os.environ.get("DEB_SOURCE")


def vendor():
    try:
        return os.environ['DEB_VENDOR']
    except KeyError:
        with open('/etc/dpkg/origins/default', 'r') as f:
            c = Deb822(f)
            return c['Vendor']


def is_debcargo_package():
    return os.path.exists('debian/debcargo.toml')


if is_debcargo_package():
    from debmutate.debcargo import DebcargoControlShimEditor, DebcargoEditor

    try:
        control = DebcargoControlShimEditor.from_debian_dir('debian')
    except AttributeError:
        control = DebcargoControlShimEditor(DebcargoEditor())
else:
    control = ControlEditor()



#!/usr/bin/python3

from lintian_brush.yaml import YamlUpdater
from lintian_brush.fixer import report_result, warn
import sys

try:
    from Levenshtein import distance
except ImportError:
    sys.exit(2)

valid_field_names = {
    'Archive',
    'ASCL-Id',
    'Bug-Database',
    'Bug-Submit',
    'Cite-As',
    'Changelog',
    'CPE',
    'Documentation',
    'Donation',
    'FAQ',
    'Funding',
    'Gallery',
    'Other-References',
    'Reference',
    'Registration',
    'Registry',
    'Repository',
    'Repository-Browse',
    'Screenshots',
    'Security-Contact',
    'Webservice',
}

typo_fixed = set()
case_fixed = set()

try:
    with YamlUpdater('debian/upstream/metadata') as updater:
        for field in updater.code:
            if field in valid_field_names:
                continue
            if (field.startswith('X-') and
                    field[2:] in valid_field_names):
                if field[2:] in updater.code:
                    warn('Both %s and %s exist.' % (
                         field, field[2:]))
                    continue
                value = updater.code[field]
                del updater.code[field]
                updater.code[field[2:]] = value
                typo_fixed.add((field, field[2:]))
                continue

            for option in valid_field_names:
                if distance(field, option) == 1:
                    value = updater.code[field]
                    del updater.code[field]
                    updater.code[option] = value
                    if option.lower() == field.lower():
                        case_fixed.add((field, option))
                    else:
                        typo_fixed.add((field, option))
                    break
except FileNotFoundError:
    pass


if case_fixed:
    kind = 'case' + ('s' if len(case_fixed) > 1 else '')
else:
    kind = ''
if typo_fixed:
    if case_fixed:
        kind += ' and '
    kind += 'typo' + ('s' if len(typo_fixed) > 1 else '')

fixed_str = ', '.join(
    ['%s => %s' % (old, new)
     for (old, new) in sorted(list(case_fixed) + list(typo_fixed))])

report_result(
    'Fix field name %s in debian/upstream/metadata (%s).' % (kind, fixed_str))



#!/usr/bin/python3

from lintian_brush.fixer import report_result, LintianIssue
from lintian_brush.lintian import dh_commands
from debmutate._rules import RulesEditor
import os
import sys
from typing import List, Tuple

if not os.path.exists('debian/rules'):
    sys.exit(2)

try:
    from Levenshtein import distance
except ModuleNotFoundError:
    sys.exit(2)

known_dh_commands = list(dh_commands())

# Include javahelper binaries, since some are just one character away from
# debhelper ones.
known_dh_commands.extend([
    'jh_build',
    'jh_classpath',
    'jh_clean',
    'jh_compilefeatures',
    'jh_depends',
    'jh_exec',
    'jh_generateorbitdir',
    'jh_installeclipse',
    'jh_installjavadoc',
    'jh_installlibs',
    'jh_linkjars',
    'jh_makepkg',
    'jh_manifest',
    'jh_repack',
    'jh_setupenvironment',
])

known_targets = []
for dh_command in known_dh_commands:
    known_targets.extend([
        'override_' + dh_command,
        'execute_before_' + dh_command,
        'execute_after_' + dh_command])

renamed: List[Tuple[str, str]] = []

with RulesEditor() as editor:
    for rule in editor.makefile.iter_all_rules():
        if rule.target.decode() in known_targets:
            continue
        for known_target in known_targets:
            issue = LintianIssue(
                'source', 'typo-in-debhelper-override-target', '%s -> %s (line X)')
            if distance(known_target, rule.target.decode()) == 1 and issue.should_fix():
                renamed.append((rule.target.decode(), known_target))
                rule.rename_target(rule.target, known_target.encode())
                issue.report_fixed()


report_result(
    'Fix typo in debian/rules rules: %s' % ', '.join('%s => %s' % (old, new) for old, new in renamed))



#!/usr/bin/python3

import sys

try:
    import asyncpg  # noqa: F401
except ModuleNotFoundError:
    sys.exit(2)

from lintian_brush.lintian_overrides import remove_unused
from lintian_brush.fixer import (
    control, net_access_allowed, report_result, diligence, fixed_lintian_tag,
    )


INTERMITTENT_LINTIAN_TAGS = [
    'rc-version-greater-than-expected-version',
    ]


if diligence() < 1:
    # Removing unused overrides requires pro-actively contacting UDD.
    sys.exit(0)

if not net_access_allowed():
    sys.exit(0)

removed = remove_unused(
    control.paragraphs, ignore_tags=INTERMITTENT_LINTIAN_TAGS)

description = [
    'Remove %d unused lintian overrides.\n' % len(removed),
    '\n',
    ]
for override in removed:
    description.append('* %s\n' % override.tag)
    fixed_lintian_tag(
        'source', 'unused-override', info=(
            override.tag, override.info if override.info else ''))

report_result(''.join(description), certainty='certain')



#!/usr/bin/python3
# Copyright (C) 2019 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Support for accessing UDD."""

import asyncio
import os


DEFAULT_UDD_URL = (
    'postgresql://udd-mirror:udd-mirror@udd-mirror.debian.net:5432/udd')


_pool = None


async def connect_udd_mirror():
    import asyncpg

    global _pool
    if not _pool:
        loop = asyncio.get_event_loop()
        udd_url = os.environ.get('UDD_URL', DEFAULT_UDD_URL)
        _pool = await asyncpg.create_pool(udd_url, loop=loop)
    return _pool.acquire()



#!/usr/bin/python3

import sys
from lintian_brush.fixer import report_result, fixed_lintian_tag

EXPECTED_URL = (
    b'https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/')

try:
    with open('debian/copyright', 'rb') as f:
        lines = list(f.readlines())
    if not lines:
        sys.exit(2)
except FileNotFoundError:
    pass
else:
    import re
    m = re.fullmatch(b'(Format|Format-Specification): (.*)', lines[0].rstrip())
    if m and (m.group(1) != b'Format' or
              m.group(2).rstrip(b'/') != EXPECTED_URL.rstrip(b'/')):
        fixed_lintian_tag(
            'source', 'unversioned-copyright-format-uri',
            m.group(2).decode())
        lines[0] = b'Format: %s\n' % EXPECTED_URL
        with open('debian/copyright', 'wb') as f:
            f.writelines(lines)

report_result("Use versioned copyright format URI.")



#!/usr/bin/python3
from lintian_brush.fixer import (
    report_result,
    opinionated,
    package_is_native,
    LintianIssue,
    is_debcargo_package,
    meets_minimum_certainty,
    warn,
    )
import os
import sys

if is_debcargo_package():
    sys.exit(0)

description = None

if not os.path.exists('debian/source/format'):
    orig_format = None
    format = '1.0'
    missing_source_format_issue = LintianIssue(
        'source', 'missing-debian-source-format')
    if not missing_source_format_issue.should_fix():
        sys.exit(0)
    missing_source_format_issue.report_fixed()
    description = "Explicitly specify source format."
else:
    with open('debian/source/format', 'r') as f:
        format = orig_format = f.read().strip()

if orig_format not in (None, '1.0'):
    sys.exit(0)

older_source_format_issue = LintianIssue(
    'source', 'older-source-format', info=(orig_format or '1.0'))

if older_source_format_issue.should_fix():
    if package_is_native():
        format = '3.0 (native)'
        description = "Upgrade to newer source format %s." % format
    else:
        from lintian_brush.patches import (
            tree_non_patches_changes,
            find_patches_directory,
            )
        from breezy import errors
        from breezy.workingtree import WorkingTree
        patches_directory = find_patches_directory('.')
        if patches_directory not in ('debian/patches', None):
            # Non-standard patches directory.
            warn(
                'Tree has non-standard patches directory %s.' % (
                    patches_directory))
        else:
            try:
                tree, path = WorkingTree.open_containing('.')
            except errors.NotBranchError as e:
                if not meets_minimum_certainty('possible'):
                    warn('unable to open vcs to check for delta: %s' % e)
                    sys.exit(0)
                format = "3.0 (quilt)"
                description = "Upgrade to newer source format %s." % format
            else:
                delta = list(tree_non_patches_changes(tree, patches_directory))
                if delta:
                    warn('Tree has non-quilt changes against upstream.')
                    if opinionated():
                        format = "3.0 (quilt)"
                        description = (
                            "Upgrade to newer source format %s." % format)
                        try:
                            with open('debian/source/options', 'r') as f:
                                options = list(f.readlines())
                        except FileNotFoundError:
                            options = []
                        if 'single-debian-patch\n' not in options:
                            options.append('single-debian-patch\n')
                            description = description.rstrip('.') + (
                                ', enabling single-debian-patch.')
                        if 'auto-commit\n' not in options:
                            options.append('auto-commit\n')
                        with open('debian/source/options', 'w') as f:
                            f.writelines(options)
                else:
                    format = "3.0 (quilt)"
                    description = "Upgrade to newer source format %s." % format

if not os.path.exists('debian/source'):
    os.mkdir('debian/source')

with open('debian/source/format', 'w') as f:
    f.write('%s\n' % format)

if format != '1.0':
    older_source_format_issue.report_fixed()

report_result(description=description)



#!/usr/bin/python
# Copyright (C) 2020 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Detect gbp dch policy."""


import os
from typing import Optional, Tuple, List

from debian.changelog import Changelog
from debmutate.changelog import (
    all_sha_prefixed,
    is_unreleased_inaugural,
    )

from breezy import osutils
from breezy.branch import Branch
from breezy.errors import NoSuchFile, RevisionNotPresent
from breezy.tree import Tree
from breezy.workingtree import WorkingTree


# Number of revisions to search back
DEFAULT_BACKLOG = 50


# TODO(jelmer): Check that what's added in the changelog is actually based on
# what was in the commit messages?


def gbp_conf_has_dch_section(tree: Tree, debian_path: str = "") -> Optional[bool]:
    try:
        gbp_conf_path = osutils.pathjoin(debian_path, "gbp.conf")
        gbp_conf_text = tree.get_file_text(gbp_conf_path)
    except NoSuchFile:
        return False
    try:
        import configparser
    except ImportError:
        return None
    else:
        parser = configparser.ConfigParser(defaults={}, strict=False)
        parser.read_string(
            gbp_conf_text.decode("utf-8", errors="replace"), gbp_conf_path
        )
        return parser.has_section("dch")


def guess_update_changelog(
    tree: WorkingTree, debian_path: str, cl: Optional[Changelog] = None
) -> Optional[Tuple[bool, str]]:
    """Guess whether the changelog should be updated.

    Args:
      tree: Tree to edit
      debian_path: Path to packaging in tree
    Returns:
      best guess at whether we should update changelog (bool)
    """
    if debian_path != "debian":
        return (
            True,
            "assuming changelog needs to be updated since "
            "gbp dch only suppors a debian directory in the root of the "
            "repository")
    changelog_path = osutils.pathjoin(debian_path, "changelog")
    if cl is None:
        try:
            with tree.get_file(changelog_path) as f:
                cl = Changelog(f)
        except NoSuchFile:
            cl = None
    if cl and is_unreleased_inaugural(cl):
        return (
            False,
            "assuming changelog does not need to be updated "
            "since it is the inaugural unreleased entry")
    ret = _guess_update_changelog_from_tree(tree, debian_path, cl)
    if ret is not None:
        return ret
    ret = _guess_update_changelog_from_branch(tree.branch, debian_path)
    if ret is not None:
        return ret
    return None


def _guess_update_changelog_from_tree(
    tree: Tree, debian_path: str, cl: Optional[Changelog]
) -> Optional[Tuple[bool, str]]:
    if gbp_conf_has_dch_section(tree, debian_path):
        return (
            False,
            "Assuming changelog does not need to be updated, "
            "since there is a [dch] section in gbp.conf.",
        )

    # TODO(jelmes): Do something more clever here, perhaps looking at history
    # of the changelog file?
    if cl:
        if all_sha_prefixed(cl[0]):
            return (
                False,
                "Assuming changelog does not need to be updated, "
                "since all entries in last changelog entry are prefixed "
                "by git shas.",
            )

    return None


def _greedy_revisions(graph, revid: bytes, length: int) -> Tuple[List[bytes], bool]:
    ret: List[bytes] = []
    it = graph.iter_lefthand_ancestry(revid)
    while len(ret) < length:
        try:
            ret.append(next(it))
        except StopIteration:
            break
        except RevisionNotPresent:
            if ret:
                ret.pop(-1)
            # Shallow history
            return ret, True
    return ret, False


def _changelog_stats(branch: Branch, history: int, debian_path: str):
    mixed = 0
    changelog_only = 0
    other_only = 0
    dch_references = 0
    with branch.lock_read():
        graph = branch.repository.get_graph()
        revids, truncated = _greedy_revisions(graph, branch.last_revision(), history)
        revs = []
        for revid, rev in branch.repository.iter_revisions(revids):
            if rev is None:
                # Ghost
                continue
            if "Git-Dch: " in rev.message:
                dch_references += 1
            revs.append(rev)
        get_deltas = branch.repository.get_revision_deltas
        for delta in get_deltas(revs):
            filenames = set(
                [a.path[1] for a in delta.added]
                + [r.path[0] for r in delta.removed]
                + [r.path[0] for r in delta.renamed]
                + [r.path[1] for r in delta.renamed]
                + [m.path[0] for m in delta.modified]
            )
            if not set(
                [
                    f
                    for f in filenames
                    if f.startswith(debian_path + '/')
                ]
            ):
                continue
            if osutils.pathjoin(debian_path, "changelog") in filenames:
                if len(filenames) > 1:
                    mixed += 1
                else:
                    changelog_only += 1
            else:
                other_only += 1
    return (changelog_only, other_only, mixed, dch_references)


def _guess_update_changelog_from_branch(
    branch: Branch, debian_path: str = "", history: int = DEFAULT_BACKLOG
) -> Optional[Tuple[bool, str]]:
    """Guess whether the changelog should be updated manually.

    Args:
      branch: A branch object
      history: Number of revisions back to analyze
    Returns:
      boolean indicating whether changelog should be updated
    """
    # Two indications this branch may be doing changelog entries at
    # release time:
    # - "Git-Dch: " is used in the commit messages
    # - The vast majority of lines in changelog get added in
    #   commits that only touch the changelog
    (changelog_only, other_only, mixed, dch_references) = _changelog_stats(
        branch, history, debian_path
    )
    if dch_references:
        return (
            False,
            "Assuming changelog does not need to be updated, since "
            "there are Gbp-Dch stanzas in commit messages",
        )
    if changelog_only == 0:
        return (
            True,
            "Assuming changelog needs to be updated, since "
            "it is always changed together with other files in the tree.",
        )
    if mixed == 0 and changelog_only > 0 and other_only > 0:
        # changelog is *always* updated in a separate commit.
        return (
            False,
            "Assuming changelog does not need to be updated, since "
            "changelog entries are always updated in separate commits.",
        )
    # Is this a reasonable threshold?
    if changelog_only > mixed and other_only > mixed:
        return (
            False,
            "Assuming changelog does not need to be updated, since "
            "changelog entries are usually updated in separate commits.",
        )
    return None


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    args = parser.parse_args()
    wt, subpath = WorkingTree.open_containing(".")
    from . import control_files_in_root
    if control_files_in_root(wt, subpath):
        debian_path = subpath
    else:
        debian_path = os.path.join(subpath, "debian")
    print(guess_update_changelog(wt, debian_path))



#!/usr/bin/python3

from debian.copyright import License
from debmutate.copyright import CopyrightEditor, NotMachineReadableError
from lintian_brush.fixer import report_result, LintianIssue

typos = {
    'bsd-2': 'BSD-2-Clause',
    'bsd-3': 'BSD-3-Clause',
    'bsd-4': 'BSD-4-Clause',
    'agpl3': 'AGPL-3',
    'agpl3+': 'AGPL-3+',
    'lgpl2.1': 'LGPL-2.1',
    'lgpl2': 'LGPL-2.0',
    'lgpl3': 'LGPL-3.0',
}
for i in [1, 2, 3]:
    typos['gplv%d' % i] = 'GPL-%d' % i
    typos['gplv%d+' % i] = 'GPL-%d+' % i
    typos['gpl%d' % i] = 'GPL-%d' % i
    typos['gpl%d+' % i] = 'GPL-%d+' % i

renames = {}


def fix_shortname(copyright):
    for paragraph in copyright.all_paragraphs():
        if paragraph.license is None:
            continue
        try:
            new_name = typos[paragraph.license.synopsis]
        except KeyError:
            continue
        issue = LintianIssue(
            'source', 'invalid-short-name-in-dep5-copyright',
            info=paragraph.license.synopsis)
        if issue.should_fix():
            renames[paragraph.license.synopsis] = new_name
            paragraph.license = License(new_name, paragraph.license.text)
            issue.report_fixed()


try:
    with CopyrightEditor() as updater:
        fix_shortname(updater.copyright)
except (FileNotFoundError, NotMachineReadableError):
    pass

report_result(
    "Fix invalid short license name in debian/copyright (%s)" % (
        ', '.join(
            ['%s => %s' % (old, new) for (old, new) in renames.items()])))



#!/usr/bin/python3

import sys

from lintian_brush.fixer import (
    control,
    LintianIssue,
    meets_minimum_certainty,
    net_access_allowed,
    report_result,
    trust_package,
    )
from typing import List
from upstream_ontologist.guess import guess_upstream_metadata

CERTAINTY = 'possible'

if not meets_minimum_certainty(CERTAINTY):
    sys.exit(0)


def textwrap_description(text) -> List[str]:
    import textwrap
    ret = []
    paras = text.split('\n\n')
    for para in paras:
        if '\n*' in para:
            ret.extend(para.splitlines())
        else:
            ret.extend(textwrap.wrap(para))
        ret.append('')
    if not ret[-1]:
        ret.pop(-1)
    return ret


def guess_description(binary_name, all_binaries, summary=None):
    if len(all_binaries) != 1:
        # TODO(jelmer): Support handling multiple binaries
        return None
    upstream_metadata = guess_upstream_metadata(
        '.', trust_package(), net_access_allowed())
    if summary is None:
        summary = upstream_metadata.get('X-Summary')
    try:
        upstream_description = textwrap_description(upstream_metadata['X-Description'])
    except KeyError:
        # Better than nothing..
        return summary
    if summary is None:
        if len(upstream_description) == 1:
            return upstream_description[0].rstrip('\n')
        return None
    lines = [line if line else '.' for line in upstream_description]
    description = summary + "\n" + ''.join([" %s\n" % line for line in lines])
    return description.rstrip('\n')


updated = []

with control as updater:
    for binary in updater.binaries:
        existing_description = binary.get('Description')
        if not existing_description:
            issue = LintianIssue(binary, 'required-field', 'Description')
            summary = None
        elif len(existing_description.strip().splitlines()) == 1:
            issue = LintianIssue(binary, 'extended-description-is-empty')
            summary = existing_description.splitlines()[0]
        else:
            continue
        if not issue.should_fix():
            continue
        description = guess_description(
            binary['Package'], list(updater.binaries), summary=summary)
        if description and description != existing_description:
            binary['Description'] = description
            updated.append(binary['Package'])
            issue.report_fixed()


report_result(
    description='Add description for binary packages: %s' %
    ', '.join(sorted(updated)),
    certainty=CERTAINTY)



#!/usr/bin/python3
# Copyright (C) 2018-2020 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Information about specific Debian/Ubuntu releases."""

import os
from debian.changelog import Version
from typing import Optional

_key_package_versions = None


def load_key_package_versions():
    import json

    path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "key-package-versions.json")
    )
    if not os.path.isfile(path):
        import pkg_resources

        path = pkg_resources.resource_filename(
            __name__, "lintian-brush/key-package-versions.json"
        )
        if not os.path.isfile(path):
            # Urgh.
            path = "/usr/share/lintian-brush/key-package-versions.json"
    with open(path, "rb") as f:
        return json.load(f)


def key_package_version(package: str, release: str) -> Optional[Version]:
    global _key_package_versions
    if _key_package_versions is None:
        _key_package_versions = load_key_package_versions()
    # TODO(jelmer): Fall back to querying UDD directly if we can't find the
    # release?
    return _key_package_versions[package].get(release)



#!/usr/bin/python3
from lintian_brush import USER_AGENT, DEFAULT_URLLIB_TIMEOUT
from lintian_brush.fixer import (
    control, net_access_allowed, report_result, LintianIssue)
import socket
import http.client
import sys
import urllib.error
import urllib.parse
from urllib.request import urlopen, Request

known_https = [
    'github.com', 'launchpad.net', 'pypi.python.org',
    'pear.php.net', 'pecl.php.net', 'www.bioconductor.org',
    'cran.r-project.org', 'wiki.debian.org']

ERRORS = (urllib.error.URLError, urllib.error.HTTPError, ConnectionResetError,
          socket.timeout, http.client.BadStatusLine)


def same_page(http_contents, https_contents):
    # This is a pretty crude way to determine we end up on the same page, but
    # it works.
    http_contents = http_contents.replace(b'https', b'').replace(b'http', b'')
    https_contents = https_contents.replace(b'https', b'').replace(
            b'http', b'')
    return http_contents == https_contents


def fix_homepage(http_url):
    if not http_url.startswith('http:'):
        return http_url
    https_url = 'https:' + http_url[len('http:'):]
    result = urllib.parse.urlparse(http_url)
    if result.netloc in known_https:
        return https_url
    if not net_access_allowed():
        return http_url
    # Fall back to just comparing the two
    headers = {'User-Agent': USER_AGENT}
    try:
        http_contents = urlopen(
            Request(http_url, headers=headers),
            timeout=DEFAULT_URLLIB_TIMEOUT).read()
    except ERRORS as e:
        sys.stderr.write(
            'Unable to access HTTP version of homepage %s: %s' %
            (http_url, e))
        return http_url
    try:
        https_contents = urlopen(
            Request(https_url, headers=headers),
            timeout=DEFAULT_URLLIB_TIMEOUT).read()
    except ERRORS as e:
        sys.stderr.write(
            'Unable to access HTTPS version of homepage %s: %s' %
            (https_url, e))
        return http_url
    if same_page(http_contents, https_contents):
        return https_url
    return http_url


try:
    with control as updater:
        try:
            homepage = updater.source["Homepage"]
        except KeyError:
            pass
        else:
            new_homepage = fix_homepage(homepage)
            if new_homepage != updater.source['Homepage']:
                issue = LintianIssue(
                    'source', 'homepage-field-uses-insecure-uri',
                    updater.source['Homepage'])
                if issue.should_fix():
                    updater.source["Homepage"] = new_homepage
                    issue.report_fixed()
except FileNotFoundError:
    sys.exit(0)


report_result("Use secure URI in Homepage field.")



#!/usr/bin/python3


from lintian_brush.fixer import report_result, LintianIssue
from debmutate._rules import (
    dh_invoke_drop_with,
    RulesEditor,
    )
from lintian_brush.patches import rules_find_patches_directory


def drop_quilt_with(line, target):
    newline = dh_invoke_drop_with(line, b'quilt')
    if line != newline:
        issue = LintianIssue(
            'source', 'dh-quilt-addon-but-quilt-source-format',
            'dh ... --with quilt (line XX)')
        if issue.should_fix():
            issue.report_fixed()
        else:
            newline = line
    return newline


try:
    with open('debian/source/format', 'r') as f:
        if f.read().strip() == '3.0 (quilt)':
            with RulesEditor() as updater:
                if rules_find_patches_directory(
                        updater.makefile) in ('debian/patches', None):
                    updater.legacy_update(command_line_cb=drop_quilt_with)
except FileNotFoundError:
    pass


report_result(
    "Don't specify --with=quilt, since package uses "
    "'3.0 (quilt)' source format.")



#!/usr/bin/python3

import os
import subprocess
import sys

from hashlib import sha1

from lintian_brush.fixer import LintianIssue, report_result

issue = LintianIssue('source', 'newer-debconf-templates', info=())

if not issue.should_fix():
    sys.exit(0)

if not os.path.isdir('debian/po'):
    sys.exit(0)


def read_hashes():
    ret = {}
    for entry in os.scandir('debian/po'):
        with open(entry.path, 'rb') as f:
            ret[entry.path] = sha1(f.read()).hexdigest()
    return ret


def debconf_updatepo():
    try:
        subprocess.check_call(['debconf-updatepo'])
    except FileNotFoundError:
        sys.exit(2)


def update_timestamp(p, ts):
    import time
    ls = []
    with open(p, 'rb') as f:
        for line in f:
            if line.startswith(b'"POT-Creation-Date: '):
                tp = time.gmtime(ts)
                line = b'"POT-Creation-Date: %s\\n"\n' % (
                    time.strftime("%Y-%m-%d %H:%M+0000", tp).encode())
            ls.append(line)
    with open(p, 'wb') as f:
        f.writelines(ls)


if 'DEBCONF_GETTEXTIZE_TIMESTAMP' in os.environ:
    old_hashes = read_hashes()
    debconf_updatepo()
    new_hashes = read_hashes()
    for p in old_hashes:
        if old_hashes[p] == new_hashes[p]:
            continue
        update_timestamp(p, int(os.environ['DEBCONF_GETTEXTIZE_TIMESTAMP']))
else:
    debconf_updatepo()


issue.report_fixed()

report_result('Run debconf-updatepo after template changes.')



#!/usr/bin/python3
# Copyright (C) 2021 Jelmer Vernooij
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Publish a package."""

from email.utils import parseaddr
import logging
import os
import sys

from breezy.controldir import ControlDir
from breezy.commit import NullCommitReporter, PointlessCommit
from breezy.errors import AlreadyBranchError, AlreadyControlDirError
from breezy.propose import iter_hoster_instances, UnsupportedHoster
from breezy.workingtree import WorkingTree
from breezy.workspace import check_clean_tree, WorkspaceDirty
from debmutate.control import ControlEditor
from debmutate.vcs import source_package_vcs

from . import get_committer
from .salsa import guess_repository_url
from .vcs import determine_browser_url


def update_control_for_vcs_url(source, vcs_type, repo_url):
    source['Vcs-' + vcs_type] = repo_url
    source['Vcs-Browser'] = determine_browser_url('git', repo_url)


class NoVcsLocation(Exception):
    """No VCS location specified or determined."""


class VcsAlreadySpecified(Exception):
    """Vcs is already specified."""


def update_offical_vcs(wt, subpath, repo_url=None, committer=None, force=False, create=False):
    # TODO(jelmer): Allow creation of the repository as well
    check_clean_tree(wt, wt.basis_tree(), subpath)

    debcargo_path = os.path.join(subpath, 'debian/debcargo.toml')
    control_path = os.path.join(subpath, 'debian/control')

    if wt.has_filename(debcargo_path):
        from debmutate.debcargo import DebcargoControlShimEditor
        editor = DebcargoControlShimEditor.from_debian_dir(wt.abspath(os.path.join(subpath, 'debian')))
    elif wt.has_filename(control_path):
        control_path = wt.abspath(control_path)
        editor = ControlEditor(control_path)
    else:
        raise FileNotFoundError(control_path)
    with editor:
        try:
            vcs_type, url = source_package_vcs(editor.source)
        except KeyError:
            pass
        else:
            if not force:
                raise VcsAlreadySpecified(vcs_type, url)
        maintainer_email = parseaddr(editor.source['Maintainer'])[1]
        source = editor.source['Source']
        if repo_url is None:
            repo_url = guess_repository_url(source, maintainer_email)
        if repo_url is None:
            raise NoVcsLocation()
        logging.info('Using repository URL: %s', repo_url)
        # TODO(jelmer): Detect vcs type in a better way
        if hasattr(wt.branch.repository, '_git'):
            vcs_type = 'Git'
        else:
            vcs_type = 'Bzr'
        update_control_for_vcs_url(editor.source, vcs_type, repo_url)

    if committer is None:
        committer = get_committer(wt)

    try:
        wt.commit(
            message='Set Vcs headers.',
            allow_pointless=False,
            reporter=NullCommitReporter(),
            committer=committer,
        )
    except PointlessCommit:
        if not force:
            # This can't happen
            raise

    if create:
        try:
            hoster, project = create_project(repo_url)
        except AlreadyControlDirError:
            logging.info('%s already exists', repo_url)
        except UnsupportedHoster:
            logging.error(
                'Unable to find a way to create %s', repo_url)
        else:
            logging.info('Created %s', repo_url)

    return repo_url


try:
    from breezy.propose import create_project
except ImportError:  # brz < 3.2
    def create_project(url):
        from urllib.parse import urlparse
        for hoster in iter_hoster_instances():
            if url.startswith(hoster.base_url):
                hoster.create_project(urlparse(url).path)
                break
        else:
            raise UnsupportedHoster(url)


def main():
    import argparse
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--identity",
        help="Print user identity that would be used when committing",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--debug", help="Describe all considerd changes.", action="store_true"
    )
    parser.add_argument(
        '--create', help='Create the repository', action='store_true')
    parser.add_argument(
        '--force', action='store_true')
    parser.add_argument(
        '--push', help='Push branch', action='store_true')

    parser.add_argument(
        "--directory",
        metavar="DIRECTORY",
        help="directory to run in",
        type=str,
        default=".",
    )
    parser.add_argument(
        'url',
        type=str,
        help='URL to publish to.',
        nargs='?')

    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    import breezy  # noqa: E402

    breezy.initialize()
    import breezy.git  # noqa: E402
    import breezy.bzr  # noqa: E402
    import breezy.plugins.gitlab

    wt, subpath = WorkingTree.open_containing(args.directory)
    if args.identity:
        print(get_committer(wt))
        return 0

    try:
        repo_url = update_offical_vcs(wt, subpath, repo_url=args.url, force=args.force, create=args.create)
    except WorkspaceDirty:
        logging.info("%s: Please commit pending changes first.", wt.basedir)
        return 1
    except NoVcsLocation:
        parser.print_usage()
        return 1
    except VcsAlreadySpecified as e:
        logging.fatal(
            'Package already in %s at %s', e.args[0], e.args[1])
        return 1
    except AlreadyBranchError as e:
        logging.fatal('Repository already exists at %s', e.path)
        return 1

    controldir = ControlDir.open(repo_url)
    try:
        branch = controldir.create_branch()
    except AlreadyBranchError:
        branch = controldir.open_branch()
    wt.branch.push(branch)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/python3

from lintian_brush.fixer import control, report_result, fixed_lintian_tag

packages = []


with control as updater:
    default_priority = updater.source.get("Priority")

    # TODO(jelmer): needs higher certainty?
    for binary in updater.binaries:
        if "transitional package" in binary.get("Description", ""):
            fixed_lintian_tag(
                binary, 'transitional-package-not-oldlibs-optional',
                '%s/%s' % (
                    binary.get('Section') or updater.source.get('Section'),
                    binary.get('Priority') or updater.source.get('Priority')))
            packages.append(binary["Package"])
            binary["Section"] = "oldlibs"
            if default_priority != "optional":
                binary["Priority"] = "optional"
            elif "Priority" in binary:
                del binary["Priority"]


report_result(
    "Move transitional package%s %s to oldlibs/optional per policy 4.0.1." %
    (("s" if len(packages) > 1 else ""), ", ".join(packages)))



#!/usr/bin/python3

import sys

from debmutate.watch import WatchEditor

from lintian_brush.fixer import report_result, LintianIssue

OBSOLETE_WATCH_FILE_FORMAT = 2
WATCH_FILE_LATEST_VERSION = 4


with WatchEditor() as editor:
    if editor.watch_file is None:
        sys.exit(0)
    if editor.watch_file.version >= WATCH_FILE_LATEST_VERSION:
        pass
    else:
        if editor.watch_file.version <= OBSOLETE_WATCH_FILE_FORMAT:
            tag = 'obsolete-debian-watch-file-standard'
        else:
            tag = 'older-debian-watch-file-standard'
        issue = LintianIssue('source', tag, '%d' % editor.watch_file.version)
        if issue.should_fix():
            editor.watch_file.version = WATCH_FILE_LATEST_VERSION
            issue.report_fixed()


report_result(
    'Update watch file format version to %s.'
    % WATCH_FILE_LATEST_VERSION)



#!/usr/bin/python3

from debmutate.deb822 import Deb822Editor
from lintian_brush.fixer import report_result, warn, fixed_lintian_tag
import sys

try:
    from Levenshtein import distance
except ImportError:
    sys.exit(2)

valid_field_names = {
    'Files', 'License', 'Copyright', 'Comment',
    'Upstream-Name', 'Format', 'Upstream-Contact',
    'Source', 'Upstream', 'Contact', 'Name'}

typo_fixed = set()
case_fixed = set()

try:
    with Deb822Editor('debian/copyright') as updater:
        for paragraph in updater.paragraphs:
            for field in paragraph:
                if field in valid_field_names:
                    continue
                if (field.startswith('X-') and
                        field[2:] in valid_field_names):
                    if field[2:] in paragraph:
                        warn('Both %s and %s exist.' % (
                             field, field[2:]))
                        continue
                    value = paragraph[field]
                    del paragraph[field]
                    paragraph[field[2:]] = value
                    typo_fixed.add((field, field[2:]))
                    fixed_lintian_tag(
                        'source', 'field-name-typo-in-dep5-copyright',
                        '%s (line XX)' % field)
                    continue

                for option in valid_field_names:
                    if distance(field, option) == 1:
                        value = paragraph[field]
                        del paragraph[field]
                        paragraph[option] = value
                        if option.lower() == field.lower():
                            case_fixed.add((field, option))
                        else:
                            typo_fixed.add((field, option))
                            fixed_lintian_tag(
                                'source', 'field-name-typo-in-dep5-copyright',
                                '%s (line XX)' % field)
                        break
except FileNotFoundError:
    pass


if case_fixed:
    kind = 'case' + ('s' if len(case_fixed) > 1 else '')
else:
    kind = ''
if typo_fixed:
    if case_fixed:
        kind += ' and '
    kind += 'typo' + ('s' if len(typo_fixed) > 1 else '')

fixed_str = ', '.join(
    ['%s => %s' % (old, new)
     for (old, new) in sorted(list(case_fixed) + list(typo_fixed))])

report_result(
    'Fix field name %s in debian/copyright (%s).' % (kind, fixed_str))



#!/usr/bin/python3

from debmutate.watch import WatchEditor

from lintian_brush.fixer import report_result, LintianIssue

# TODO(jelmer): Remove dh_make pattern:

TEMPLATE_PATTERNS = [
    r'^\s*#\s*(Example watch control file for uscan)'
    ]


with WatchEditor() as editor:
    if editor.watch_file:
        for entry in editor.watch_file.entries:
            try:
                filenamemangle = entry.get_option('filenamemangle')
            except KeyError:
                continue
            if (filenamemangle ==
                    r's/.+\/v?(\d\S+)\.tar\.gz/<project>-$1\.tar\.gz/'):
                issue = LintianIssue(
                    'source', 'debian-watch-contains-dh_make-template',
                    '<project>')
                if issue.should_fix():
                    entry.del_option('filenamemangle')
                    issue.report_fixed()


report_result('Remove dh_make template from debian watch.')



#!/usr/bin/python3

import re
from lintian_brush.fixer import report_result, fixed_lintian_tag
from debmutate._rules import update_rules, Makefile
from typing import Set, Tuple, Optional

pkg_info_included = False
PATH = '/usr/share/dpkg/pkg-info.mk'
INCLUDE_LINE = b'include ' + PATH.encode('ascii')

_KNOWN = [
    (b'dpkg-parsechangelog | sed -n -e \'s/^Version: //p\'',
     'DEB_VERSION'),
]

_variables: Set[str] = set()
message = None

var_sre = re.compile(b'([A-Z_]+)[ \t]*([:?]?=)[ \t]*(.*)')


def variable_defined(var):
    global _variables
    if _variables:
        return var in _variables
    with open(PATH, 'rb') as f:
        for line in f:
            m = var_sre.match(line.strip())
            if m:
                _variables.add(m.group(1).decode())
    return var in _variables


def is_pkg_info_var_line(line: str) -> Tuple[
        Optional[str], bool, bool, Optional[bool]]:
    m = var_sre.match(line.strip())
    if not m:
        return (None, False, False, None)
    variable = m.group(1).decode()
    hard_assignment = m.group(2) != b"?="
    assignment = m.group(3)
    if not variable_defined(variable):
        return (variable, False, False, hard_assignment)
    m = re.match(b'\\$\\(shell (.*)\\)', assignment)
    if not m:
        return (variable, True, False, hard_assignment)
    cmd = m.group(1)
    for r, v in _KNOWN:
        if r == cmd:
            return (v, True, True, hard_assignment)
    else:
        return (variable, True, False, hard_assignment)


def drop_pkg_info_line(line):
    global pkg_info_included
    if line.strip() == INCLUDE_LINE:
        pkg_info_included = True
        return line
    (variable, variable_matches,
     value_matches, hard) = is_pkg_info_var_line(line)
    if not variable_matches:
        return line
    if not value_matches:
        # The assignment is different; let's include
        # the mk file *before* this assignment
        if not pkg_info_included and variable_matches:
            pkg_info_included = True
            return [INCLUDE_LINE, line]
        else:
            return line
    if hard:
        lineno = -1  # TODO(jelmer): Pass this up
        fixed_lintian_tag(
            'source', 'debian-rules-parses-dpkg-parsechangelog',
            info='%s (line %d)' % (variable, lineno))
    if not pkg_info_included:
        pkg_info_included = True
        return INCLUDE_LINE
    return None


try:
    mf = Makefile.from_path('debian/rules')
except FileNotFoundError:
    pass
else:
    for line in mf.contents:
        if (isinstance(line, bytes) and
                is_pkg_info_var_line(line)[1:3] == (True, True)):
            update_rules(global_line_cb=drop_pkg_info_line)
            break

report_result('Avoid invoking dpkg-parsechangelog.')



#!/usr/bin/python3
from debmutate.control import drop_dependency
from debmutate.debhelper import ensure_minimum_debhelper_version
from lintian_brush.fixer import control, report_result, LintianIssue


with control as updater:
    for field in ["Build-Depends", "Build-Depends-Indep",
                  "Build-Depends-Arch"]:
        try:
            old_build_depends = updater.source[field]
        except KeyError:
            continue
        issue = LintianIssue(
            updater.source, 'build-depends-on-obsolete-package',
            info='%s: %s' % (field.lower(), 'dh-systemd'))
        if not issue.should_fix():
            continue
        updater.source[field] = drop_dependency(
            old_build_depends, "dh-systemd")
        if old_build_depends != updater.source[field]:
            ensure_minimum_debhelper_version(updater.source, "9.20160709")
            issue.report_fixed()


report_result(
    "Depend on newer debhelper (>= 9.20160709) rather than dh-systemd.")



#!/usr/bin/python3

from debmutate.control import ensure_some_version
from lintian_brush.fixer import (
    report_result,
    control,
    LintianIssue,
    )
from lintian_brush.line_editor import LineEditor
import os
import re

resolution = ""


def update_configure_ac(path):
    global resolution
    if not os.path.exists(path):
        return

    with LineEditor(path, 'b') as e:
        for lineno, line in e:
            m = re.fullmatch(
                b'\\s*AC_PATH_PROG\\s*'
                b'\\(\\s*(\\[)?(?P<variable>[A-Z_]+)(\\])?\\s*'
                b',\\s*(\\[)?pkg-config(\\])?\\s*'
                b'(,\\s*(\\[)?(?P<default>.*)(\\])?\\s*)?\\)\n', line)

            if not m:
                continue

            issue = LintianIssue(
                'source',
                'autotools-pkg-config-macro-not-cross-compilation-'
                'safe', info='%s (line %d)' % (name, lineno))
            if not issue.should_fix():
                continue

            if (m.group('variable') == b'PKG_CONFIG' and
                    not m.group('default')):
                e[lineno] = b'PKG_PROG_PKG_CONFIG\n'
                resolution = (
                    "This patch changes it to use "
                    "PKG_PROG_PKG_CONFIG macro from pkg.m4.")
                # Build-Depend on pkg-config for pkg.m4
                with control:
                    control.source['Build-Depends'] = (
                        ensure_some_version(
                            control.source.get('Build-Depends', ''),
                            'pkg-config'))
            else:
                e[lineno] = line.replace(b'AC_PATH_PROG', b'AC_PATH_TOOL')
                resolution = (
                    "This patch changes it to use AC_PATH_TOOL.")

            issue.report_fixed()


for name in ['configure.ac', 'configure.in']:
    update_configure_ac(name)


report_result(
    """Use cross-build compatible macro for finding pkg-config.

The package uses AC_PATH_PROG to discover the location of pkg-config(1). This
macro fails to select the correct version to support cross-compilation.

%s

Refer to https://bugs.debian.org/884798 for details.
""" % resolution,
    patch_name='ac-path-pkgconfig')



#!/usr/bin/python3

import argparse
import subprocess
import sys

from lintian_brush import available_lintian_fixers

from ruamel.yaml import YAML

KNOWN_KEYS = ['tag', 'status', 'difficulty', 'comment']

all_tags = set([
    tag.decode()
    for tag in
    subprocess.check_output(
        ['lintian-explain-tags', '--list-tags']).splitlines()])

supported_tags = set()
for fixer in available_lintian_fixers():
    supported_tags.update(fixer.lintian_tags)

yaml = YAML()
with open('tag-status.yaml', 'r') as f:
    tag_status = yaml.load(f)

per_tag_status = {}
for entry in tag_status or []:
    per_tag_status[entry['tag']] = entry
    extra_keys = set(entry.keys()) - set(KNOWN_KEYS)
    assert not extra_keys, 'Unknown keys: %r' % extra_keys


for tag in supported_tags:
    existing = per_tag_status.get(tag)
    if existing and existing.get('status') != 'implemented':
        raise Exception(
            'tag %s is marked as %s in tag-status.yaml, but implemented' %
            (tag, existing.get('status')))
    per_tag_status[tag] = {'status': 'implemented'}


parser = argparse.ArgumentParser()
parser.add_argument(
    '--new-tags', action='store_true', help='List missing tags.')
parser.add_argument(
    '--check', action='store_true', help='Check tags.')
args = parser.parse_args()

if args.new_tags:
    for tag in sorted(all_tags):
        if tag not in per_tag_status:
            print(tag)
elif args.check:
    retcode = 0
    for tag in sorted(all_tags):
        if tag not in per_tag_status:
            print('Missing tag: %s' % tag)
            retcode = 1
    sys.exit(retcode)



#!/usr/bin/python3

from debmutate.control import drop_dependency
from lintian_brush.fixer import control, report_result, LintianIssue


with control as updater:
    for field in ['Build-Depends', 'Build-Depends-Indep']:
        try:
            updater.source[field] = drop_dependency(
                updater.source[field], "build-essential")
        except KeyError:
            pass
        else:
            issue = LintianIssue(
                updater.source, 'build-depends-on-build-essential',
                field)
            # TODO(jelmer): Check overrides
            issue.report_fixed()


report_result("Drop unnecessary dependency on build-essential.")



#!/usr/bin/python3

# Extract renamed tags from lintian metadata.

from debian.deb822 import Deb822
import json
import os


renames = {}


def read_desc_files(path):
    for entry in os.scandir(path):
        if entry.is_dir():
            read_desc_files(entry.path)
        elif entry.name.endswith('.tag'):
            with open(entry.path, 'r') as f:
                desc = Deb822(f)
                for renamed_from in desc.get('Renamed-From', '').splitlines():
                    if renamed_from.strip():
                        renames[renamed_from.strip()] = desc['Tag']


read_desc_files('/usr/share/lintian/tags/')


with open('renamed-tags.json', 'w') as f:
    json.dump(renames, f, indent=4, sort_keys=True)



#!/usr/bin/python3

from debmutate.watch import WatchEditor

from lintian_brush.fixer import report_result
from lintian_brush.watch import fix_old_github_patterns


with WatchEditor() as updater:
    fix_old_github_patterns(updater)


report_result(
    "Update pattern for GitHub archive URLs from /<org>/<repo>/tags page"
    "/<org>/<repo>/archive/<tag> -> /<org>/<repo>/archive/refs/tags/<tag>."
)



#!/usr/bin/python3

from lintian_brush.fixer import (
    control,
    meets_minimum_certainty,
    report_result,
    fixed_lintian_tag,
    )
from lintian_brush.section import (
    get_name_section_mappings,
    find_expected_section,
    )
import sys

CERTAINTY = 'likely'

if not meets_minimum_certainty(CERTAINTY):
    sys.exit(0)

try:
    regexes = get_name_section_mappings()
except FileNotFoundError:
    # lintian not installed?
    sys.exit(2)


default_section = None
fixed = []

with control as updater:
    default_section = updater.source.get("Section")

    for binary in updater.binaries:
        expected_section = find_expected_section(regexes, binary["Package"])
        section = binary.get("Section", default_section)
        if expected_section and expected_section != section:
            fixed.append(
                ('binary package %s' % binary["Package"],
                 section, expected_section))
            binary["Section"] = expected_section
            fixed_lintian_tag(
                binary, 'wrong-section-according-to-package-name',
                info='%s => %s' % (binary['Package'], expected_section))


# TODO(jelmer): If there is only a single binary package without section, just
# set the section of the source package?
report_result(
    "Fix sections for %s." % ', '.join(['%s (%s => %s)' % v for v in fixed]),
    certainty=CERTAINTY)
